/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Name:				slx3_brightpass.fx																													//
// Programmierer:	Sebastian Leopold - Kontakt: Sebsatian Leopold @ aol.com																	//
// Copyright:		Copyright  2005 by Sebastian Leopold																							//
//-------------------------------------------------------------------------------------------------------------------------//
// Beschreibung:	Filtert alle Hellen stellen aus einem bild und macht sie wei.															//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*

	Weitere Informationen:
	
	Dieser Effekt ist ein Post - Processing effekt und besteht aus diesem Grund nur aus einem Pixelshader. Bei jedem der
	Post - Processing Effekte befindet sich die Original Scene in der zweiten Post - Texture. Dies ist auch der Fall wenn
	es sich um den ersten Effekt handelt, dann ist jedoch der erste und der zweie Slot gleich.
	
	Dieser Effekt filtert die Besondershellen stellen eines Bildes heraus und frbt sie wei ein. Dazu sind Pixel Shader 2.0
	von nten. Alle anderen Bereiche werden schwarz eingefrbt.
	
*/

// Texturen
texture postTex1;				// Post - Processing Texture ( enthlt das aktuelle Scenen Bild )
texture postTex2;				// Post - Processing Texture 2 ( enhlt immer das original Scenen Bild in Original Auflsung)
texture effTex1;				// Effekt Textur ( zustzliche Texture fr Spezial Effekte)	
texture effTex2;				// Effekt Textur ( zustzliche Texture fr Spezial Effekte)

// Variablen
float4 vecValues1;			// Enthlt die ersten 4 benutzerdefinierbaren Effektwerte
float4 vecValues2;			// Enthlt die zweiten 4 benutzerdefinierbaren Effektwerte

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Sampler fr Texturen in diesem Effekt
sampler currentScene = sampler_state
{
	texture 		= (postTex1);
	MinFilter	= linear;
	MagFilter	= linear;
	MipFilter	= linear;
	AddressU		= Clamp;
	AddressV		= Clamp;
};


float Luminance = 0.08f;
static const float fMiddleGray = 0.18f;
static const float fWhiteCutoff = 0.8f;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Fragmentprogramm fr Pixelberechnung
float4 BrightPass_PS(float2 texcoord0 : TEXCOORD0) : COLOR
{
	// Aktuellen Pixel samplen:
	float3 pixel = tex2D(currentScene,texcoord0);
	
	pixel *= fMiddleGray / (Luminance + 0.001f);
	pixel *= (1.0f + (pixel / (fWhiteCutoff * fWhiteCutoff)));
	pixel -= 5.0f;
	
	pixel = max(pixel,0.0f);
	pixel /= (10.0f + pixel);
	return float4(pixel,1.0f);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Technik 0 fr Brightpass Effekt
technique tech_00
{
	pass pass_00
	{
		// Effekt compillieren
		Pixelshader = compile ps_2_0 BrightPass_PS();
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Fallback Funktion --> No Effect
technique tech_01
{
	pass pass_00
	{
		Texture[0] = <postTex1>;
	}
}