////////////////////////////////////////////////////////////////////////////////////////////////////////
// Name:							sd_water.fx																						//
// Programmierer:				Sebastian Leopold																				//
// Copyright:					Copyright 2005 by Sebastian Leopold													//
//----------------------------------------------------------------------------------------------------//
// Beschreibung:				HLSL Wassershader konzipiert und erstellt fr Sunset Down mit der Sylex 3.0//
//									3D Grafikerweiterung und Der Acknex 6.4 Engine. Der Effekt beinhaltet 		//
//									sowohl Refraction als auch Causticen und Reflection								//
////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////
// ALLGEMEIN
////////////////////////////////////////////////////////////////////////////////////////////////////////

// Texturen
texture entTex0;
texture entTex1;
texture entTex2;

// Matrizen
float4x4 matWorldViewProjection;
float4x4 matWorldView;
float4x4 matTextureReflection;
float4x4 matTextureRefraction;
float4x4 matBumpWave;

// Variablen
float4 vecSkill2;
float4 vecSkill3;

// Tempore Variablen
float 	Dtu;
float 	Dtv;
float2 	BumpTex;
float3	CameraSpacePos;
float 	InvCameraSpacePosZ;
float4	t1;

////////////////////////////////////////////////////////////////////////////////////////////////////////
// VERTEXSHADER
////////////////////////////////////////////////////////////////////////////////////////////////////////

// Struktur fr Vertexshader Ausgabe des
// High - Quality Wasserfragmentprogramms
struct VS_OUTPUT_ReflectAndRefract
{
	float4 Pos							: POSITION;
	float2 Tex0							: TEXCOORD0;
	float3 CameraSpacePosRefract	: TEXCOORD1;
	float3 CameraSpacePosReflect	: TEXCOORD2;
};

// Fragmentprogramm des Effekts ( Vertexfragmentprogramm )
VS_OUTPUT_ReflectAndRefract VS_ReflectAndRefract(float4 Pos : POSITION, float2 Tex0 : TEXCOORD0)
{
	// Ausgabe Variable definieren und initialisieren
	VS_OUTPUT_ReflectAndRefract Output = (VS_OUTPUT_ReflectAndRefract)0;
	
	// Vertexposition im Bildraum berechnen
	Output.Pos = mul(Pos,matWorldViewProjection);
	
	// Texturkoordinaten in Stage0 einfach weiterreichen
	Output.Tex0 = Tex0;
	
	// Vertexposition im Kameraraum berechnen
	CameraSpacePos = mul(Pos,(float4x3)matWorldView);
	
	// Texturkoordinaten im Kameraraum
	Output.CameraSpacePosRefract = mul(CameraSpacePos,(float4x3)matTextureRefraction);
	Output.CameraSpacePosReflect = mul(CameraSpacePos,(float4x3)matTextureReflection);
	
	return Output;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
// PIXELSHADER
////////////////////////////////////////////////////////////////////////////////////////////////////////

// Texturesampler
sampler ReflectionSampler = sampler_state
{
	texture 		= (entTex1);
	AddressU		= Wrap;
	AddressV		= Mirror;
	MipFilter 	= linear;
	MinFilter 	= linear;
	MagFilter	= linear;
};


sampler RefractionSampler = sampler_state
{
	texture 		= (entTex0);
	AddressU		= Wrap;
	AddressV		= Wrap;
	MipFilter 	= linear;
	MinFilter 	= linear;
	MagFilter	= linear;
};

// Texturesampler
sampler Bumpmap = sampler_state
{
	texture 		= (entTex2);
	AddressU		= Wrap;
	AddressV		= Wrap;
	MipFilter 	= linear;
	MinFilter 	= linear;
	MagFilter	= linear;
};

// Fragmentprogramm des Effekts ( Pixelfragmentprogramm )
float4 PS_ReflectAndRefract(float2 Tex0 : TEXCOORD0, float3 CameraSpacePosRefract : TEXCOORD1, float3 CameraSpacePosReflect : TEXCOORD2) : COLOR
{	
	// nderung der Texturkoordinaten berechnen
	t1 = tex2D(Bumpmap,Tex0);
	Dtu = t1.x * matBumpWave._11 + t1.y * matBumpWave._21;
	Dtv = t1.x * matBumpWave._12 + t1.y * matBumpWave._22;

	// Projizierte Texturkoordinaten berechnen
	InvCameraSpacePosZ			= 1.0f / CameraSpacePosReflect.z;
	CameraSpacePosReflect.x		= CameraSpacePosReflect.x * InvCameraSpacePosZ;
	CameraSpacePosReflect.y		= CameraSpacePosReflect.y * InvCameraSpacePosZ;
	
	BumpTex.x = clamp(CameraSpacePosReflect.x + Dtu * vecSkill3.y,0.01f,0.99f);
	BumpTex.y = clamp(CameraSpacePosReflect.y + Dtv * vecSkill3.y,-0.99f,-0.01f);
	
	t1 = tex2D(ReflectionSampler,BumpTex);

	InvCameraSpacePosZ 			= 1.0f / CameraSpacePosRefract.z;
	CameraSpacePosRefract.x 	= CameraSpacePosRefract.x * InvCameraSpacePosZ;
	CameraSpacePosRefract.y 	= CameraSpacePosRefract.y * InvCameraSpacePosZ;
	
	BumpTex.x = clamp(CameraSpacePosRefract.x + 0.55 * Dtu * vecSkill3.x,-0.99f,-0.01f);
	BumpTex.y = clamp(CameraSpacePosRefract.y + 0.55 * Dtv * vecSkill3.x, 0.01f, 0.99f); 

	return (t1 * vecSkill3.z) + ((tex2D(RefractionSampler,BumpTex) * vecSkill2 * vecSkill3.w));
}

// Fragmentprogramm des Effekts( Nur Refraction )
float4 PS_Refract(float2 Tex0 : TEXCOORD0, float3 CameraSpacePosRefract : TEXCOORD1) : COLOR
{
	// nderung der Texturkoordinaten berechnen
	t1 = tex2D(Bumpmap,Tex0);
	Dtu = t1.x * matBumpWave._11 + t1.y * matBumpWave._21;
	Dtv = t1.x * matBumpWave._12 + t1.y * matBumpWave._22;
	
	InvCameraSpacePosZ 			= 1.0f / CameraSpacePosRefract.z;
	CameraSpacePosRefract.x 	= CameraSpacePosRefract.x * InvCameraSpacePosZ;
	CameraSpacePosRefract.y 	= CameraSpacePosRefract.y * InvCameraSpacePosZ;
	
	BumpTex.x = clamp(CameraSpacePosRefract.x + 0.55 * Dtu,-0.99f,-0.01f);
	BumpTex.y = clamp(CameraSpacePosRefract.y + 0.55 * Dtv, 0.01f, 0.99f); 

	return (tex2D(RefractionSampler,BumpTex) * vecSkill2 * 1.0);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
// EFFEKT
////////////////////////////////////////////////////////////////////////////////////////////////////////
technique ReflectAndRefract
{
	pass pass1
	{
		FogEnable = FALSE;
		VertexShader = compile vs_2_0 VS_ReflectAndRefract();
		PixelShader = compile ps_2_0 PS_ReflectAndRefract();
	}
}

technique Refract
{
	pass pass1
	{
		FogEnable = True;
		FogVertexMode = linear;
		FogTableMode = linear;
		Cullmode  = CW;
		VertexShader = compile vs_2_0 VS_ReflectAndRefract();
		PixelShader = compile ps_2_0 PS_Refract();
	}
}